#include "Slicer.h"
#include "HMath.h"
#include "Ppm.h"

#include <iostream>

using namespace Math;

GLuint 
SimpleSlicer::
slice(Basis& basis, int w, int h, double F, double x, double y, double z)
{
	GLuint tex;
	glGenTextures(1,&tex);
	glBindTexture(GL_TEXTURE_2D,tex);
	
	GLfloat* img = new GLfloat[w*h];

	double dt;
	if (w<h) dt = F/h;
	else dt = F/w;

	int i,j;
	for(i=0;i<h;i++)
	{
		for(j=0;j<w;j++)
		{			
			double dx = dt*j;
			double dy = dt*i;			

			img[i*w+j] = basis(x+dx,y+dy,z);
		}
	}
	
	glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE,w,h,0,GL_LUMINANCE,GL_FLOAT,img);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);

	delete img;

	return tex;
}

GLuint 
SimpleSlicer::
slice(ColourBasis& basis, int w, int h, double F, double x, double y, double z)
{
	GLuint tex;
	glGenTextures(1,&tex);
	glBindTexture(GL_TEXTURE_2D,tex);

	GLfloat* img = new GLfloat[w*h*4];

	double dt;
	if (w<h) dt = F/h;
	else dt = F/w;

	int i,j;
	for(i=0;i<h;i++)
	{
		for(j=0;j<w;j++)
		{			
			double dx = dt*j;
			double dy = dt*i;			

			GLfloat* where = &img[(i*w+j)*4];
			Colour c = basis(x+dx,y+dy,z);
			c.clamp();
			
			where[0] = c.r;
			where[1] = c.g;
			where[2] = c.b;
			where[3] = c.a;
		}
	}
	
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,w,h,0,GL_RGBA,GL_FLOAT,img);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);

	delete img;

	return tex;
}


void
SimpleSlicer::
sliceToFile(string file, Basis& basis, int w, int h, double F, double x, double y, double z)
{
	unsigned char *data = new unsigned char[w*h*3];

	double dt;
	if (w<h) dt = F/h;
	else dt = F/w;

	int i,j;
	for(i=0;i<h;i++)
	{
		for(j=0;j<w;j++)
		{			
			double dx = dt*j;
			double dy = dt*i;	
			double b = basis(x+dx,y+dy,z);

			if (b<0) b = 0;
			else if (b>1) b = 1;
			
 			unsigned char c = (unsigned char)(255*b);
			data[(i*w+j)*3] = c;
			data[(i*w+j)*3+1] = c;
			data[(i*w+j)*3+2] = c;
		}
	}
	
	Ppm::Save(data,w,h,file);
	delete data;
}

void 
SimpleSlicer::
sliceToFile(string file, ColourBasis& basis, int w, int h, double F, double x, double y, double z)
{
	unsigned char *data = new unsigned char[w*h*3];

	double dt;
	if (w<h) dt = F/h;
	else dt = F/w;

	int i,j;
	for(i=0;i<h;i++)
	{
		for(j=0;j<w;j++)
		{			
			double dx = dt*j;
			double dy = dt*i;			

			unsigned char* where = &data[(i*w+j)*3];
			Colour c = basis(x+dx,y+dy,z);
			c.clamp();			
					
			where[0] = (unsigned char) (255 * c.r);
			where[1] = (unsigned char) (255 * c.g);
			where[2] = (unsigned char) (255 * c.b);
		}
	}
	
	Ppm::Save(data,w,h,file);
	delete data;
}




















GLuint 
SphereSlicer::
slice(Basis& basis, int w, int h, double R, double x0, double y0, double z0)
{
	GLuint tex;
	glGenTextures(1,&tex);
	glBindTexture(GL_TEXTURE_2D,tex);
	
	GLfloat* img = new GLfloat[w*h];

	double iw = 1./w;
	double ih = 1./h;

	int i,j;
	for(i=0;i<h;i++)
	{
		for(j=0;j<w;j++)
		{			
			double theta = iw*j*TWO_PI;
			double phi = ih*i*PI;			

			double x = R*cos(theta)*sin(phi);
			double y = R*sin(theta)*sin(phi);
			double z = R*cos(phi);

			img[i*w+j] = basis(x0+x,y0+y,z0+z);
		}
	}
	
	glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE,w,h,0,GL_LUMINANCE,GL_FLOAT,img);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);

	delete img;

	return tex;
}

GLuint 
SphereSlicer::
slice(ColourBasis& basis, int w, int h, double R, double x0, double y0, double z0)
{
	GLuint tex;
	glGenTextures(1,&tex);
	glBindTexture(GL_TEXTURE_2D,tex);
	
	GLfloat* img = new GLfloat[w*h*4];

	double iw = 1./w;
	double ih = 1./h;
	
	int i,j;
	for(i=0;i<h;i++)
	{
		for(j=0;j<w;j++)
		{			
			double theta = iw*j*TWO_PI;
			double phi = ih*i*PI;			

			double x = R*cos(theta)*sin(phi);
			double y = R*sin(theta)*sin(phi);
			double z = R*cos(phi);
	
			GLfloat* where = &img[(i*w+j)*4];
			Colour c = basis(x0+x,y0+y,z0+z);
			c.clamp();
			
			where[0] = c.r;
			where[1] = c.g;
			where[2] = c.b;
			where[3] = c.a;

		}
	}
	
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,w,h,0,GL_RGBA,GL_FLOAT,img);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);

	delete img;

	return tex;
}


