#ifndef SLICER_H
#define SLICER_H

/* Slicer: Takes a slice through the space of a Basis function 
 *   and binds it to a GL Texture object
 * BP 20.08.05
 */

#include "Basis.h"
#include "ColourBasis.h"
#include <GL/gl.h>
#include <string>

/* SimpleSlicer: Takes a simple rectangular slice (in the z-plane) 
 *   through basis, with texture width W, height H, and magnitude F,
 *   starting at (x,y,z) */
class SimpleSlicer
{
	public:
	
	static GLuint slice(Basis& b, int w, int h, double F = 1, double x = 0, double y = 0, double z = 0);
	static GLuint slice(ColourBasis& b, int w, int h, double F = 1, double x = 0, double y = 0, double z = 0);

	static void sliceToFile(std::string file, Basis& b, int w, int h, double F = 1, double x = 0, double y = 0, double z = 0);
	static void sliceToFile(std::string file, ColourBasis& b, int w, int h, double F = 1, double x = 0, double y = 0, double z = 0);
};

/* SphereSlicer: Takes a slicer on the surface of a parameterised sphere
 *   The sphere is parameterised by theta in [0,2pi] and phi in [0,pi]
 */

class SphereSlicer
{
	public:
	
	static GLuint slice(Basis& b, int w, int h, double R = 1, double x = 0, double y = 0, double z = 0);
	static GLuint slice(ColourBasis& b, int w, int h, double R = 1, double x = 0, double y = 0, double z = 0);
};


#endif
