#include "Vector3.h"

#include <cmath>

Vector3::Vector3()
{
	mData[0]=mData[1]=mData[2]=mData[3]=1;	
}

Vector3::Vector3(double x, double y, double z, double w)
{
	mData[0] = x;
	mData[1] = y;
	mData[2] = z;
	mData[3] = w;
}

void Vector3::setData(double x, double y, double z, double w)
{
	mData[0]=x;
	mData[1]=y;
	mData[2]=z;
	mData[3]=w;
}

Vector3 Vector3::cross(const Vector3& a, const Vector3& b)
{
	double x,y,z;

	x = a.y()*b.z()-a.z()*b.y();
	y = a.z()*b.x()-a.x()*b.z();
	z = a.x()*b.y()-a.y()*b.x();

	return Vector3(x,y,z);
}

double Vector3::dot(const Vector3& v1, const Vector3& v2)
{
	return v1.x()*v2.x()+v1.y()*v2.y()+v1.z()*v2.z();
}

Vector3 Vector3::operator*(double d) const
{
	return Vector3(x()*d,y()*d,z()*d,w());
}

Vector3 Vector3::operator*(const Vector3& v) const
{
	return Vector3(x()*v.x(),y()*v.y(),z()*v.z());
}

Vector3 Vector3::operator+(const Vector3& v) const
{
	return Vector3(mData[0]+v.x(),mData[1]+v.y(),mData[2]+v.z(),1);
}

Vector3 Vector3::add(const Vector3& a, const Vector3& b)
{
	return Vector3(a.x()+b.x(),a.y()+b.y(),a.z()+b.z(),a.w()+b.w());
}

Vector3 Vector3::operator-(const Vector3& v) const
{
	return Vector3(mData[0]-v.x(),mData[1]-v.y(),mData[2]-v.z(),1);
}

Vector3& Vector3::operator+=(const Vector3& v)
{
	mData[0]+=v.x();
	mData[1]+=v.y();
	mData[2]+=v.z();
	return *this;	
}

Vector3& Vector3::operator-=(const Vector3& v)
{
	mData[0]-=v.x();
	mData[1]-=v.y();
	mData[2]-=v.z();
	return *this;	
}

Vector3& Vector3::normalise()
{
	double oneOnL = 1/length();
	mData[0]*=oneOnL;
	mData[1]*=oneOnL;
	mData[2]*=oneOnL;
	mData[3] = 1;
	return *this;
}

double Vector3::length()
{
	return std::sqrt(mData[0]*mData[0]+mData[1]*mData[1]+mData[2]*mData[2]);
}

void Vector3::print(std::ostream& out)
{
	out << "<" 	<< mData[0] << ","
			<< mData[1] << ","
			<< mData[2] << ">";
}

void Vector3::printx(std::ostream& out)
{
	out << "<" 	<< mData[2] << ","
			<< mData[1] << ","
			<< mData[0] << ">";
}
