/* A 3d Vector class (implicitly homogenous)
 * BP
 */

#ifndef VECTOR3_H
#define VECTOR3_H

#include <ostream>


class Vector3
{
	private:

	double mData[4];

	public:

	Vector3();
	Vector3(double x, double y, double z, double w=1);

	/* Accessors */
	double* getData(){return mData;}
	double x() const {return mData[0];}
	double y() const {return mData[1];}
	double z() const {return mData[2];}
	double w() const {return mData[3];}
	double& operator[](int i){return mData[i];}

	void setData(double x, double y, double z, double w=1);

	/* Operations */

	Vector3 operator-() const {return Vector3(-x(),-y(),-z(),-w());}
	Vector3 operator+(const Vector3& v) const;
	Vector3 operator-(const Vector3& v) const;
	Vector3 operator*(double d) const;
	Vector3 operator*(const Vector3& v) const;
	Vector3& operator+=(const Vector3& v);
	Vector3& operator-=(const Vector3& v);

	/* Procedures */

	Vector3& normalise(); //normalises the vector
	double length(); //returns the length

	static Vector3 cross(const Vector3& v1, const Vector3& v2);
	static double dot(const Vector3& v1, const Vector3& v2);
	static Vector3 add(const Vector3& v1, const Vector3& v2);
	/* Printable */

	void print(std::ostream& out);
	void printx(std::ostream& out); // print with x and z swapped (for povray)
};

#endif
