#ifndef SUPER_H
#define SUPER_H

#include <GL/gl.h>
#include <cstdlib>
#include <string>

class SuperFormula
{
	double mParams[6];
	int mDivs;
	bool mCached;
	GLuint mListId;

	double mRotation;
	double mRotationSpeed;

	double mFactor;
	
	GLfloat mColours[12]; // outside, center, iris
	double mBorderThickness; // thickness of black border
	double mEyeSize;

	double mSpeed; // speed of movement
	double mRecklessness; // tendency to change direction (0 no change, 1 most change)

	double mDirection[2];
	double mPosition[2];

	double mNextBlink;
	bool mBlinking;

	static const int sParamRanges[9][2];
	static const double sParamMultiplier[9];

		void drawEye();
		void fillCircle();
		void rotate(double angle);
		double toroidal_clamp(double x, double leastx, double mostx);
		static double randD(double from = 0, double to = 1);
		void buildList(); // build display list
		double& getParam(int param);

	public:
		SuperFormula(double a, double b, double m, double n1, double n2, double n3, int divisions = 0);
		~SuperFormula();
		SuperFormula(const SuperFormula& sf);

		void setParam(int param, int val);
	public:
		void draw();
		void drawPreview(); // without translation transforms..
		void recache(){mCached = false;}
		void update(double dt); // update animation...
	
		double x(){return mPosition[0];}
		double y(){return mPosition[1];}
		double x(double x){return mPosition[0]=x;}
		double y(double y){return mPosition[1]=y;}

		// 0<=param<3, 0<=rgba<4
		double getColour(int param, int rgba){return mColours[param*4+rgba];}
		void setColour(int param, double r, double g, double b, double a)
		{
			GLfloat* col = mColours+(param*4);
			col[0] = r; col[1] = g; col[2] = b; col[3] = a;
			recache();
		}

		int getSliderVal(int param){return (int)(getParam(param)/sParamMultiplier[param]);} // normalised for slider, 0<=param<=8

		// create and return a (slightly mutated) child close by
		SuperFormula* spawn(double mutationFactor = 0.01);
		static SuperFormula* random(); // return a randomly generated superformula creature (assumes rand has been seeded)
		
		static void getRange(int param, int& low, int& high){low = sParamRanges[param][0]; high = sParamRanges[param][1];}
		static std::string getParamName(int param);
};

#endif
