#include "superviewbox.h"

#include <QtGui>
#include <QString>

#include "glwidget.h"

	SuperViewBox::SuperViewBox(SuperFormula* sf, GLWidget* glw)
	:mParent(glw)
{
	mView = new SuperView(sf);
	mInsert = new QPushButton("insert");
	
	connect(mInsert,SIGNAL(pressed(void)),this,SLOT(insert(void)));

	for(int i=0;i<9;i++)
	{
		int low,high;
		SuperFormula::getRange(i,low,high);
		mParameterSliders[i] = makeSlider(low,high);
		if (sf!=NULL)
			mParameterSliders[i]->setValue(sf->getSliderVal(i));
		mParameterLabels[i] = new QLabel(QString::fromStdString(SuperFormula::getParamName(i)));
	}
	
	connect(mParameterSliders[0],SIGNAL(valueChanged(int)),this,SLOT(setParam0(int)));
	connect(mParameterSliders[1],SIGNAL(valueChanged(int)),this,SLOT(setParam1(int)));
	connect(mParameterSliders[2],SIGNAL(valueChanged(int)),this,SLOT(setParam2(int)));
	connect(mParameterSliders[3],SIGNAL(valueChanged(int)),this,SLOT(setParam3(int)));
	connect(mParameterSliders[4],SIGNAL(valueChanged(int)),this,SLOT(setParam4(int)));
	connect(mParameterSliders[5],SIGNAL(valueChanged(int)),this,SLOT(setParam5(int)));
	connect(mParameterSliders[6],SIGNAL(valueChanged(int)),this,SLOT(setParam6(int)));
	connect(mParameterSliders[7],SIGNAL(valueChanged(int)),this,SLOT(setParam7(int)));
	connect(mParameterSliders[8],SIGNAL(valueChanged(int)),this,SLOT(setParam8(int)));

	col0 = new QPushButton("colour 0");
	col1 = new QPushButton("colour 1");
	col2 = new QPushButton("colour 2");

	connect(col0,SIGNAL(pressed(void)),this,SLOT(setColour0(void)));
	connect(col1,SIGNAL(pressed(void)),this,SLOT(setColour1(void)));
	connect(col2,SIGNAL(pressed(void)),this,SLOT(setColour2(void)));

	// layout
	QHBoxLayout* layout = new QHBoxLayout;

	// left side
	QWidget* left = new QWidget;
	QVBoxLayout* leftSide = new QVBoxLayout;
	leftSide->addWidget(mView);
	leftSide->addWidget(mInsert);
	left->setLayout(leftSide);

	// right side
	QWidget* right = new QWidget;
	QVBoxLayout *rights = new QVBoxLayout;

	for(int i=0;i<9;i++)	
	{
		QWidget* param = new QWidget;
		QHBoxLayout* hbl = new QHBoxLayout;
	
		hbl->addWidget(mParameterLabels[i]);
		hbl->addWidget(mParameterSliders[i]);

		param->setLayout(hbl);
		rights->addWidget(param);
	}
	rights->addWidget(col0);
	rights->addWidget(col1);
	rights->addWidget(col2);
	right->setLayout(rights);

	// finished
	layout->addWidget(left);
	layout->addWidget(right);
	setLayout(layout);
	setWindowTitle(tr("SuperFormula Viewer"));

	// start animation
	show();
	mView->timer();
}

SuperViewBox::~SuperViewBox()
{
	delete mInsert;
	for(int i=0;i<9;i++)
		delete mParameterSliders[i];
	
	delete col0;
	delete col1;
	delete col2;

	delete mView;
}

QSlider* SuperViewBox::makeSlider(int min, int max)
{
	QSlider* slider = new QSlider(Qt::Horizontal);
	slider->setRange(min,max);
	slider->setSingleStep(1);
	slider->setTickInterval(1+(max-min)/10);
	slider->setTickPosition(QSlider::TicksBelow);
	return slider;
}

void SuperViewBox::insert()
{
	if (mParent and mView->sf()) 
	{
		mView->sf()->x(0);
		mView->sf()->y(0);
		mParent->addNewThing(new SuperFormula(*mView->sf()));
	}
}

void SuperViewBox::setColour(int i)
{
	SuperFormula* sf = mView->sf();
	if (!sf) return;

	int r = (int)(255*sf->getColour(i,0));
	int g = (int)(255*sf->getColour(i,1));
	int b = (int)(255*sf->getColour(i,2));
	int a = (int)(255*sf->getColour(i,3));

	QRgb initial = QColor(r,g,b,a).rgba();
	bool okay;

	QRgb result = QColorDialog::getRgba(initial,&okay);

	if (okay)
	{
		//set new colour
		sf->setColour(i,qRed(result)/255.0,qGreen(result)/255.,qBlue(result)/255.,qAlpha(result)/255.);
	}
	// else // do nothing
}
